<template>
	<div class="layout-parent">
		<router-view v-slot="{ Component }" :key="key">
			<transition :name="setTransitionName" mode="out-in">
				<keep-alive :include="getKeepAliveNames">
					<component :is="Component" :key="refreshRouterViewKey" class="w100" v-show="!isIframePage" />
				</keep-alive>
			</transition>
		</router-view>
		<transition :name="setTransitionName" mode="out-in">
			<Iframes class="w100" v-show="isIframePage" :refreshKey="iframeRefreshKey" :name="setTransitionName"
				:list="iframeList" />
		</transition>
	</div>
</template>

<script setup lang="ts" name="layoutParentView">
import { ref, reactive, toRef, toRefs, onMounted, getCurrentInstance, computed, onBeforeMount, nextTick, onUnmounted, watch, defineAsyncComponent } from 'vue'
import { useThemeConfig } from '/@/stores/themeConfig'
import { storeToRefs } from 'pinia';
import { useKeepALiveNames } from '/@/stores/keepAliveNames'
import { useRoute, useRouter } from 'vue-router'
import { Session } from '/@/utils/storage';
import mittBus from '/@/utils/mitt';
const Iframes = defineAsyncComponent(() => import('/@/layout/routerView/iframes.vue'))

interface ParentViewState {
	refreshRouterViewKey: string;
	iframeRefreshKey: string;
	keepAliveNameList: string[];
	iframeList: string[];
}
const storesThemeConfig = useThemeConfig()
const { themeConfig } = storeToRefs(storesThemeConfig)
const storesKeepAliveNames = useKeepALiveNames()
const { keepAliveNames, cachedViews } = storeToRefs(storesKeepAliveNames)
const state = reactive<ParentViewState>({
	refreshRouterViewKey: '', // 非 iframe tagsview 右键菜单刷新时
	iframeRefreshKey: '', // iframe tagsview 右键菜单刷新时
	keepAliveNameList: [],
	iframeList: [],//返回路由所有iframe的数组
})
const { refreshRouterViewKey, iframeRefreshKey, iframeList } = toRefs(state)
const route = useRoute()
const router = useRouter()
//获取router-view的key,刷新路由
const key = computed(() => {
	// alert(route.fullPath)
	console.log('key11', route.fullPath)
	return route.fullPath;//刷新路由
})
// 设置主界面切换动画
const setTransitionName = computed(() => {
	return themeConfig.value.animation
})
// 获取组件缓存列表(name值)
const getKeepAliveNames = computed(() => {
	return themeConfig.value.isTagsview ? cachedViews.value : state.keepAliveNameList
	// return [];//设置页面路由不缓存，返回include空数组
})
// 设置 iframe 显示/隐藏
const isIframePage = computed(() => {
	return route.meta.isIframe
})
// 获取 iframe 组件列表(未进行渲染)
const getIframeListRoutes = async () => {
	console.log('router11', router.getRoutes())//获取所有路由
	console.log(router.getRoutes())
	router.getRoutes().forEach((v: any) => {
		if (v.meta.isIframe) {
			v.meta.isIframeOpen = false
			v.meta.loading = true
			state.iframeList.push({ ...v })//返回路由所有iframe的数组
		}
	})
	// console.log('state.iframeList', state.iframeList)
}
// 页面加载前，处理缓存，页面刷新时路由缓存处理--tagsview右键点击’刷新‘传递过来的事件，这边on接收onTagsViewRefreshRouterView---》刷新当前 tagsView
onBeforeMount(() => {
	state.keepAliveNameList = keepAliveNames.value
	mittBus.on('onTagsViewRefreshRouterView', (fullPath: string) => {//刷新当前 tagsView
		state.keepAliveNameList = keepAliveNames.value.filter((name: string) => route.name !== name)//刷新的原理：先获取其他缓存的数据
		state.refreshRouterViewKey = ''
		state.iframeRefreshKey = ''
		nextTick(() => {
			state.refreshRouterViewKey = fullPath
			state.iframeRefreshKey = fullPath
			state.keepAliveNameList = keepAliveNames.value//刷新的原理：先获取其他缓存的数据，再获取所有的数据
		})
	})
})
// 页面加载时
onMounted(() => {
	getIframeListRoutes()

	/**
	 * 页面缓存在刷新之后失效
	原因是cachedViews在刷新时没有值，也就造成原本已经打开的tagView不会正确的放入缓存中，
	建议在parent.vue中onBeforeMount钩子函数加载时将Session中的tagsViewList取出来赋值给cachedViews。
	同样也会存在一个问题，刷新打开的页面，第一次切换不会正确缓存，第二次切换才能够正常缓存
 */

	nextTick(() => {
		setTimeout(() => {
			if (themeConfig.value.isCacheTagsView) {
				let tagsViewArr: RouteItem[] = Session.get('tagsViewList') || []
				cachedViews.value = tagsViewArr.filter((item) => item.meta?.isKeepAlive).map((item) => item.name as string)
			}
		}, 0);
	});

})
// 页面卸载时
onUnmounted(() => {
	mittBus.off('onTagsViewRefreshRouterView', () => { })
})
// 监听路由变化，防止 tagsView 多标签时，切换动画消失
//fix: bug refreshRouterViewKey值为null 导致路由缓存第一次无效,增加{ immediate: true }
// console.log('route.fullPath', route.fullPath)
// console.log('decodeURI', decodeURI(route.fullPath))
watch(() => route.fullPath, () => { state.refreshRouterViewKey = decodeURI(route.fullPath) }, { immediate: true })//immediate:true表示立即加载
</script>
<style lang='scss' scoped></style>